﻿using System;
using System.Activities;
using System.Activities.DurableInstancing;
using System.Runtime.DurableInstancing;
using System.Threading;

namespace Microsoft.Samples.Activities
{
    //-----------------------------------------
    // note: running this code requires setting up
    //       a WF4.5 instance store
    //-----------------------------------------
    class Program
    {        
        // change this flag to run the code with and without identity
        static bool useIdentity = false;

        static void Main()
        {        
            // start an instance, unload it, and persist it 
            // into the instance store
            Guid instanceId = StartAndUnloadInstance();

            // load the instance with the wrong definition 
            LoadAndCompleteInstanceWithWrongDefinition(instanceId);
                        
            Console.WriteLine("Press [Enter] to exit.");
            Console.ReadLine();
        }

        static Guid StartAndUnloadInstance()
        {
            ShowMessageFromHost("Starting instance", ConsoleColor.Green);

            // ****************************************
            // Create the workflow application
            // ****************************************
            WorkflowApplication application = null;
            if (useIdentity)
            {
                // with identity             
                WorkflowIdentity identityV1 = new WorkflowIdentity("WorkflowIdentitySample", new Version(1, 0, 0, 0), null);
                application = new WorkflowApplication(new Activity1(), identityV1);
                ShowMessageFromHost(string.Format("Creating with identity '{0}'", application.DefinitionIdentity.ToString())); 
            }
            else
            {
                application = new WorkflowApplication(new Activity1());
                ShowMessageFromHost("Creating without identity");
            }
        
            // configure the workflow application and start executing            
            AutoResetEvent syncEvent = new AutoResetEvent(false);
            application.InstanceStore = SetupInstanceStore();
            application.PersistableIdle = (e) => PersistableIdleAction.Unload;
            application.Unloaded = (e) =>
                {
                    ShowMessageFromHost("Persisted and unloaded from memory");
                    syncEvent.Set();
                };
            application.Run();

            syncEvent.WaitOne();

            return application.Id;
        }

        static void LoadAndCompleteInstanceWithWrongDefinition(Guid instanceId)
        {
            string input = Console.ReadLine();

            try
            {
                ShowMessageFromHost("Reloading instance");
        
                //   We are attempting to load the current instance
                //   using a different version of the program definition!!!        
                WorkflowApplication application = null;
                if (useIdentity)
                {
                    // with identity:
                    WorkflowIdentity identityV2 = new WorkflowIdentity("WorkflowIdentitySample", new Version(2, 0, 0, 0), null);
                    application = new WorkflowApplication(new Activity2(), identityV2);
                    ShowMessageFromHost(string.Format("Loading with identity '{0}'", application.DefinitionIdentity.ToString()));
                }
                else
                {
                    // without identity:
                    application = new WorkflowApplication(new Activity2());
                }

                // configure the workflow application
                AutoResetEvent syncEvent = new AutoResetEvent(false);
                application.InstanceStore = SetupInstanceStore();
                application.Unloaded = (e) => syncEvent.Set();

                // load the application (will fail due to version mismatch)                
                application.Load(instanceId);

                // this resumes the bookmark setup by readline           
                application.ResumeBookmark("ReadLine", input);
                syncEvent.WaitOne();
            }
            catch (VersionMismatchException ex)
            {
                ShowMessageFromHost(string.Format("A {0} occurred.", ex.GetType().Name), ConsoleColor.Red);
                ShowMessageFromHost(string.Format("Message: {0}", ex.Message), ConsoleColor.Red);
            }
            catch (Exception ex)
            {        
                ShowMessageFromHost(string.Format("A {0} occurred.", ex.GetType().Name), ConsoleColor.Red);
                ShowMessageFromHost(string.Format("   Message: {0}", ex.Message), ConsoleColor.Red);
            }
        }
 
        static InstanceStore SetupInstanceStore()
        {
            InstanceStore instanceStore = new SqlWorkflowInstanceStore(@"Data Source=.\SQLExpress;Initial Catalog=InstanceStore;Integrated Security=True");

            InstanceHandle handle = instanceStore.CreateInstanceHandle();
            InstanceView view = instanceStore.Execute(handle, new CreateWorkflowOwnerCommand(), TimeSpan.FromSeconds(30));
            handle.Free();

            instanceStore.DefaultInstanceOwner = view.InstanceOwner;

            return instanceStore;
        }

        static void ShowMessageFromHost(string message, ConsoleColor color = ConsoleColor.Yellow)
        {
            var currentColor = Console.ForegroundColor;
            Console.ForegroundColor = color;
            Console.WriteLine("[Host] {0}", message);
            Console.ForegroundColor = currentColor;
        }
    }   
}